<?php

namespace Drupal\travis_build\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\travis_build\TravisBuildBot;

/**
 * Configure maintenance settings for this site.
 *
 * @internal
 */
class SettingsForm extends ConfigFormBase {

  /**
   * The Travis build bot service.
   *
   * @var \Drupal\travis_build\TravisBuildBot
   */
  private $buildBot;

  /**
   * Constructs a \Drupal\system\ConfigFormBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \Drupal\travis_build\TravisBuildBot $travis_build_bot
   *   The Travis build bot service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, TravisBuildBot $travis_build_bot) {
    parent::__construct($config_factory);
    $this->buildBot = $travis_build_bot;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('travis_build.bot')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'travis_build_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['travis_build.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('travis_build.settings');

    $status = $this->buildBot->getBuildStatus();

    $form['status'] = [
      '#type' => 'details',
      '#open' => TRUE,
      '#title' => $this->t('Build status'),
    ];
    $form['status'][] = [
      '#markup' => '<h2>' . $this->t('@status', ['@status' => $status]) . '</h2>',
    ];

    $form['repo_url'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Repo URL'),
      '#description' => $this->t('Github repo URL of the format <code>organization-name/repo-name</code>.'),
      '#default_value' => $config->get('repo_url'),
    ];

    $form['branch'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Branch or tag'),
      '#description' => $this->t('Enter which branch or tag to trigger.'),
      '#default_value' => $config->get('branch'),
    ];

    $form['auth_token'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Auth Token'),
      '#description' => $this->t('Enter the authorization token. It should not include the word "token".'),
      '#default_value' => $config->get('auth_token'),
    ];

    $form['throttle'] = [
      '#type' => 'number',
      '#title' => $this->t('Update frequency'),
      '#description' => $this->t('How often the module will trigger a build and query build status, in seconds. Default value is every 5 minutes.'),
      '#default_value' => $config->get('throttle') ?? 300,
    ];

    $form['actions']['trigger_build'] = [
      '#type' => 'submit',
      '#value' => $this->t('Trigger build now'),
      '#id' => 'edit-trigger-build-now',
      '#weight' => 2,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('travis_build.settings')
      ->set('repo_url', $form_state->getValue('repo_url'))
      ->set('branch', $form_state->getValue('branch'))
      ->set('auth_token', $form_state->getValue('auth_token'))
      ->set('throttle', $form_state->getValue('throttle'))
      ->save();

    if ($form_state->getTriggeringElement()['#id'] == 'edit-trigger-build-now') {
      $this->buildBot->triggerBuild($right_now = TRUE);
    }
    else {
      parent::submitForm($form, $form_state);
    }
  }

}
