<?php

namespace Drupal\travis_build;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Drupal\Core\State\StateInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use GuzzleHttp\ClientInterface;
use GuzzleHttp\Exception\RequestException;

/**
 * Trigger a build and query build status.
 *
 * @ingroup travis_build
 */
class TravisBuildBot {

  use MessengerTrait;
  use StringTranslationTrait;

  /**
   * The Travis API url, with organization and repo name.
   */
  const API_BASE_URL = 'https://api.travis-ci.com/repo/';

  /**
   * Which build statuses are considered completed.
   */
  const COMPLETED_STATUSES = [
    'canceled',
    'errored',
    'failed',
    'passed',
  ];

  /**
   * The config factory service.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  private $configFactory;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  private $state;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * Construct a new TravisBuildBot instance.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state service.
   * @param \GuzzleHttp\ClientInterface $http_client
   *   The HTTP client.
   * @param \Drupal\Core\StringTranslation\TranslationInterface $translation
   *   The translation service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, StateInterface $state, ClientInterface $http_client, TranslationInterface $translation, MessengerInterface $messenger) {
    $this->configFactory = $config_factory;
    $this->state = $state;
    $this->httpClient = $http_client;
    $this->setMessenger($messenger);
  }

  /**
   * Fetch API endpoint.
   *
   * @param string $method
   *   The HTTP request method: GET, POST, or DELETE.
   * @param string $path
   *   The API path, which will be prefixed with the base url.
   * @param array $options
   *   Request options to pass to Guzzle request.
   */
  protected function apiRequest(string $method, string $path, array $options = []) {
    $config = $this->configFactory->get('travis_build.settings');
    $repo_url = $config->get('repo_url');
    if (!$repo_url) {
      return NULL;
    }
    // Sanitize the repo url - convert / to %2F and drop trailing slash.
    $repo_url = str_replace('/', '%2F', rtrim($repo_url, '/'));

    $url = self::API_BASE_URL . $repo_url . $path;

    try {
      $default_options = [
        'verify' => FALSE,
        'headers' => [
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'Travis-API-Version' => '3',
          'Authorization' => 'token ' . $config->get('auth_token'),
        ],
      ];
      // Merge passed in options with defaults.
      if (!empty($options['json']) && !empty($default_options['json'])) {
        $options['json'] += $default_options['json'];
      }
      $options += $default_options;
      $response = $this->httpClient->request($method, $url, $options);

      // Update last request time for throttling.
      $this->state->set('travis_build.last_request', \Drupal::time()->getRequestTime());
    }
    catch (RequestException $e) {
      watchdog_exception('travis_build', $e);
      return NULL;
    }

    return Json::decode($response->getBody());
  }

  /**
   * Returns whether sufficient time has passed to make an API request.
   */
  protected function noMoreWaiting() {
    $current_time = \Drupal::time()->getRequestTime();
    $last_request = $this->state->get('travis_build.last_request');
    $throttle = $this->configFactory->get('travis_build.settings')->get('throttle') ?? 300;

    return $current_time > $last_request + $throttle;
  }

  /**
   * Get the current build status.
   */
  public function getBuildStatus() {
    $config = $this->configFactory->get('travis_build.settings');
    $branch = $config->get('branch');

    if (!$branch) {
      return NULL;
    }

    $state = $this->state->get('travis_build.status');
    if (!$this->noMoreWaiting() || in_array($state, self::COMPLETED_STATUSES)) {
      return $state;
    }

    if ($state == 'pending') {
      // Process a pending build.
      $state = $this->processBuild();
      return $state;
    }

    // Build has already been sent. Get build status.
    $response = $this->apiRequest('GET', '/builds', [
      'query' => [
        'branch.name' => $branch,
        'limit' => 1,
      ],
    ]);

    if (!empty($response['builds'])) {
      $current_build = $response['builds'][0];

      $state = $current_build['state'];
      $this->state->set('travis_build.status', $state);
      return $state;
    }

    return NULL;
  }

  /**
   * Trigger a build.
   *
   * @param bool $right_now
   *   Whether or not to push a build immediately and cancel previous builds.
   */
  public function triggerBuild($right_now = FALSE) {
    $this->state->set('travis_build.status', 'pending');

    if ($right_now || $this->noMoreWaiting()) {
      $this->processBuild();
    }

    return TRUE;
  }

  /**
   * Process a pending build.
   */
  public function processBuild() {
    $config = $this->configFactory->get('travis_build.settings');
    $branch = $config->get('branch');
    $json_body = ['request' => ['branch' => $branch]];
    $response = $this->apiRequest('POST', '/requests', ['json' => $json_body]);
    $state = $this->state->set('travis_build.status', 'sent');
    return $state;
  }

}
